import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class ControlCanvas extends ScaleCanvas implements MouseListener {
    Manager M;
    RenderJulia JD; 
    ControlPanelColor DISPLAY,POLY;
    HandleMap MC;  
    ListenSquare EXPORT;
    PopupManager POPUP;
    ListenSquare RESET;

     public ControlCanvas() {
	 addMouseListener(this);
	 JD=new RenderJulia();
	 MC=new HandleMap(0,225);
	 POPUP=new PopupManager(this.M,80,225);
	 RESET=new ListenSquare(0,375,76,30);
	 setPanels();
     }



    public void setPanels() {  
	 Color[] C0={new Color(100,0,100),Color.white,
         Color.white,Color.white,new Color(180,180,180)};
	 Color[] C1={new Color(0,0,255),Color.white,
         Color.white,Color.white,new Color(180,180,180)};


	 String[] DisplayString={"frame","pointer","core",
               "partition","polygon","star convex classes","main display"};
	 Color[] DisplayColor={new Color(255,120,0),Color.white,
                   new Color(0,255,200,200),
                               Color.white,Color.yellow,Color.green};
	 int[] DisplayState={1,0,0,1,0,0};
         DISPLAY=new ControlPanelColor(C1,DisplayString,DisplayState,6,DisplayColor);
	 DISPLAY.mode=0;

	 String[] PolyString={"DOMAIN fill","DOMAIN outline",
              "DOMAIN image fill","DOMAIN image outline",
              "CONTAINER fill","CONTAINER outline","auxiliary polygon display"};
	 Color[] PolyColor={new Color(200,0,200,150),
         Color.white,new Color(255,0,0,100),Color.white,
          new Color(100,100,100,100),Color.white};
	 int[] PolyState={1,1,1,1,1,1};
         POLY=new ControlPanelColor(C0,PolyString,PolyState,6,PolyColor);
	 POLY.mode=0;


    }


    public Color getColor() {
	try{return(M.K.SC.C);}
	catch(Exception e) {return(Color.red);}
    }

   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawControls(g);
      drawInstructions(g);	
   }

   public void drawBG(Graphics2D g) {
        g.setColor(new Color(40,70,255));
	g.setColor(new Color(0,0,100));
        g.fillRect(0,0,getWidth(),getHeight()); 
	g.setColor(Color.white); 
        g.drawRect(0,0,getWidth()-1,getHeight()-1); 
   }

    public void drawControls(Graphics2D g) {
        DISPLAY.render(g,0,0,160);
	g.translate(0,-3);
	g.setColor(new Color(80,160,255));
        g.setFont(new Font("Helvetica",Font.PLAIN,12));
	g.drawString("To get started,",5,130);
	g.drawString("or when in doubt,",5,145);
	g.drawString("click on the rectangle",5,160);
	g.drawString("that says INSTRUCTIONS",5,175);
	g.drawString("Also, the TUTORIAL button",5,190);
	g.drawString("summons a window that",5,205);
	g.drawString("gives some tutorials.",5,220);
	g.translate(0,3);


        if(DISPLAY.L[4].on==1) POLY.render(g,0,110,160);	
	MC.render(g);	
	POPUP.render(g);
	RESET.render(g,"RESET",20,5,new Color(200,0,0));
    }

    public void drawInstructions(Graphics2D g) {
	g.translate(0,20);
        g.setFont(new Font("Helvetica",Font.PLAIN,12));
	g.drawString("keyboard commands",5,420);
	g.drawString("z: zoom in",5,445);
	g.drawString("x: select point",5,460);
	g.drawString("c: zoom out",5,475);
	g.drawString("1234567890ab+-",5,500);
	g.drawString("change polygon vertex",5,515);
	g.drawString("Use these buttons",5,530);
	g.drawString("in tandem with the",5,545);
	g.drawString("poly select window.",5,560);
	g.translate(0,-20);

    }


    /**This is turned on at the beginning
       of the program and otherwise never activated.
       It draws the Julia set.*/
    public boolean doRun() {
	if(JD.HALT==false) return(false);
	JD=new RenderJulia(this.M);
        new Thread(JD).start();
	return(true);
    }

    public void documentDisplay(int d) {
	if(d==99) Documentation.displayInfo(this.M);
	if(d==0) Documentation.frameInfo(this.M);
	if(d==1) Documentation.pointerInfo(this.M);
	if(d==2) Documentation.coreInfo1(this.M);
	if(d==3) Documentation.triInfo(this.M);	
	if(d==4) DocumentPoly.info(this.M);
	if(d==5) Documentation.starInfo(this.M);
    }

    public void documentPoly(int d) {
	if(d!=-1) DocumentPoly.polyDisplay(this.M);
    }


    public void mouseClicked(MouseEvent e) {
        MouseData J=MouseData.process(e);	

	int d=DISPLAY.process(J.X,getColor());
	documentDisplay(d);
	d=MC.process(J.X);
	if(d==99) Documentation.mapInfo(this.M);
	if(d==-99) Documentation.specialMapInfo(this.M);

	d=POLY.process(J.X,getColor());	
	documentPoly(d);
	POPUP.M=this.M;
	POPUP.process(J.X);
	if(RESET.inside(J.X)==1) reset();
	M.repaint();
    }

     public void mousePressed(MouseEvent e) {}
     public void mouseReleased(MouseEvent e) {}
     public void mouseEntered(MouseEvent e) {}
     public void mouseExited(MouseEvent e) {}   


    /**RESET STUFF*/

    public void reset() {
	M.P.TR.COUNT=0;
	M.P.TR.INDEX=-1;
	try{M.DB.X.HALT=true;}
	catch(Exception e){}
	try{M.TC.QIT.HALT=true;}
	catch(Exception e){}
	M.P.setScales(188,408,20);
        PlotStyle.DENS=4;  //partition piece plot density
        PlotStyle.WIDTH=1; //partition piece outline width
        PlotStyle.BOLD=4;  //julia set plot contrast
        PlotStyle.AFFINE=0; //use canonical affine scale: 0=no,1=yes
        PlotStyle.ZOOM=4;   //strength of zooming in/out
	double phi=(1+Math.sqrt(5))/2;
	double d=Math.pow(phi,5)/2;
	double e=Math.pow(2,-3);
	M.P.SCALE=Math.pow(d,e);
	setPanels();
    }




}

