import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class DebugComposition {


    /**checks the linear functional routine.  We know that
       the map (1,0) carries the point (phi+1,y) onto the
       line x=phi^6.  Let L be this line.  We compose the
       map (1,0) with the linear functional (1,0,-phi^6)
       and check that it vanishes for y= i + j phi where
       i and j both range from 0 to 4.*/

    public static void test0() {
	System.out.println("linear functional composition check. should get zeros");
	QI2 A=new QI2(1,0);
	QI2 B=new QI2(0,0);
	QI2 C=new QI2(0,1);
	C=QI2.pow(C,6);
	C=C.negate();

	QI2RationalMap X=DataMap.map(1,0);
	QI2RationalFunction P=X.composeWithFunctional(A,B,C);
	for(int i=0;i<5;++i) {
	for(int j=0;j<5;++j) {
	   QI2 Y=P.P.evaluate(new QI2(1,1),new QI2(i,j));
	   Y.println();
	}}
    }





    /**checks the composition with affine functionals*/

    public static void test1() {
	System.out.println("affine precomposition test");
	QI2 A10=Debugger.randomQI2();
	QI2 A01=Debugger.randomQI2();
	QI2 A00=Debugger.randomQI2();
	QI2 B10=Debugger.randomQI2();
	QI2 B01=Debugger.randomQI2();
	QI2 B00=Debugger.randomQI2();
	QI2Polynomial P=Debugger.randomCubic2();
	QI2Polynomial Q=P.composeWithAffine(A10,A01,A00,B10,B01,B00);
	QI2 X=Debugger.randomQI2();
	QI2 Y=Debugger.randomQI2();
	QI2 GOAL=Q.evaluate(X,Y);

	QI2 LX=QI2.plus(A00,QI2.plus(QI2.times(A10,X),QI2.times(A01,Y)));
	QI2 LY=QI2.plus(B00,QI2.plus(QI2.times(B10,X),QI2.times(B01,Y)));
	QI2 TRY=P.evaluate(LX,LY);
	System.out.println("compare");
	GOAL.println();
	TRY.println();
    }

    /**This builds on test5. The way it works is that we compose the 
       polynomial from test5 with the linear map which carries the
       standard triangle (0,0) (1,0) (0,1) to the triangle with vertices
       (1,1) (phi,1) (phi,phi).  The line x+y=1, which is the diagonal
       of the standard triangle, is carried to the line on which the
       poly from test5 vanishes.  So, our new poly should vanish on
       the line x+y=1.  We check this on 25 random inputs.*/

    public static void test2() {
	System.out.println("holistic check. should get zeros");
	QI2 A=new QI2(1,0);
	QI2 B=new QI2(0,0);
	QI2 C=new QI2(0,1);
	C=QI2.pow(C,6);
	C=C.negate();

	QI2RationalMap X=DataMap.map(1,0);
	QI2RationalFunction P=X.composeWithFunctional(A,B,C);

	QI2[] AA={new QI2(1,0),new QI2(1,0)};
	QI2[] BB={new QI2(1,1),new QI2(1,0)};
	QI2[] CC={new QI2(1,1),new QI2(1,1)};

	QI2Polynomial PP=P.P.moveTriangle(AA,BB,CC);
	for(int i=0;i<5;++i) {
	for(int j=0;j<5;++j) {
	    QI2 Y=PP.evaluate(new QI2(1,0),new QI2(i,j));
	    Y.println();
	}}
    }


    /**Checks that the square-trianble transform works*/

    public static void test3() {
	System.out.println("check square-triangle transform");
	QI2Polynomial P=Debugger.randomCubic2();
	QI2Polynomial Q=P.squareTriangleTransform();
	QI2[] x1={Debugger.randomQI2(),Debugger.randomQI2()};
	QI2[] y1={x1[0],QI2.times(x1[0],x1[1])};
	QI2 x2=P.evaluate(y1);
	QI2 y2=Q.evaluate(x1);
	System.out.println("compare");
	x2.println();
	y2.println();
    }


    /**checks the confine operation. Given a rational map f and a
       line L determined by a pair of points A,B, this routine produces
       a polynomial which vanishes on points p such that f(p) in L.
       Here we just check this for the identity map.*/

    public static void test4() {
	System.out.println("check confine construction");
	QI2[] A={Debugger.randomQI2(),Debugger.randomQI2()};
	QI2[] B={Debugger.randomQI2(),Debugger.randomQI2()};
	QI2RationalMap F=QI2RationalMap.identityMap();
	QI2Polynomial P=F.confine(A,B);
	P.print();
	QI2 AA=P.evaluate(A);
	QI2 BB=P.evaluate(B);
	System.out.println("should be 0s");
	AA.println();
	BB.println();
    }



}