import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

/**This is our class for members of Z[phi]*/

public class QI {
    BigInteger a;
    BigInteger b;     // a + b phi


    /**These are the construction methods*/
    public QI(long A,long B) {
	Long A1=new Long(A);
	Long B1=new Long(B);
	this.a=new BigInteger(A1.toString());
	this.b=new BigInteger(B1.toString());
    }

    public QI() {
	this.a=new BigInteger("0");
	this.b=new BigInteger("0");
    }

    public QI(QI x) {
	this.a=x.a;
	this.b=x.b;
    }

    public QI(BigInteger A) {
	this.a=A;
	this.b=new BigInteger("0");
    }


    /**The arithmetic operations*/

    public static QI plus(QI x,QI y) {	
        QI z=new QI();
	z.a=x.a.add(y.a);
	z.b=x.b.add(y.b);
	return(z);
    }

    public static QI minus(QI x,QI y) {	
        QI z=new QI();
	z.a=x.a.subtract(y.a);
	z.b=x.b.subtract(y.b);
	return(z);
    }

    public static QI times(QI x,QI y) {  
        QI z=new QI();
	z.a=x.a.multiply(y.a);
	z.a=z.a.add(x.b.multiply(y.b));
	z.b=x.a.multiply(y.b);
	z.b=z.b.add(x.b.multiply(y.a));
	z.b=z.b.add(x.b.multiply(y.b));
	return(z);
    }

    public static QI pow(QI x,int j) {
	QI y=new QI(1,0);
	for(int i=0;i<j;++i) {
	    y=times(y,x);
	}
	return(y);
    }

    public QI negate() {
	return(QI.minus(new QI(0,0),this));
    }

    /**The sign*/

    public boolean isZero() {
	BigInteger ZERO=new BigInteger("0");
	int A=a.compareTo(ZERO);
	if(A!=0) return(false);
	int B=b.compareTo(ZERO);
	if(B!=0) return(false);
	return(true);
    }
       
    public boolean isPositive() {
	BigInteger TWO=new BigInteger("2");
	BigInteger aa=TWO.multiply(a);
	aa=aa.add(b);
	return(QISign.isPositive(aa,b));
    }

    public boolean isPositiveOrZero() {
	if(isZero()==true) return(true);
	if(isPositive()==true) return(true);
	return(false);
    }

    public int sign() {
	if(isZero()==true) return(0);
	boolean test1=isPositive();
	if(test1==true) return(1);
	QI X=this.negate();
	boolean test2=X.isPositive();
	if(test2==true) return(-1);
	return(99);
    }


    /**Here are routines governing the interaction
       between QIs and doubles.*/


    /**This routine finds a Z[phi] approximation to a
       double, within the specified accuracy.  We try
       to take the one with smallest coefficients.
       If the tolerance is too small, the program won't
       work.  However, for our purposes, we don't need
       that much accuracy.*/

    public static QI approx(double d,double t) {
	double f=Math.floor(d);
	long k=(long)(f);
	double e=d-f;
	QI A=approxRaw(e,t);
	A=plus(A,new QI(k,0));
	return(A);
    }

    public static QI approxRaw(double d,double t) {
	for(int k=0;k<6;++k) {
	    int lim=(int)(Math.pow(10,k));
	    for(int j=-lim;j<lim;++j) {
		int l1=-(int)(Math.floor(j*phi()))-2;
		int l2=l1+5;
		for(int i=l1;i<l2;++i) {
		    QI D=new QI(i,j);
		    double e=D.toDouble();
		    double test=Math.abs(d-e);
		    if(test<t) return(D);
		}
	    }
	}
	return(null);
    }

    /**conversion*/

    public BigDecimal toBigDecimal() {
	BigDecimal a2=new BigDecimal(a.toString());
	BigDecimal b2=new BigDecimal(b.toString());
	BigDecimal c=a2.add(b2.multiply(PHI()));
	return(c);
    }

    public double toDouble() {
	BigDecimal c=this.toBigDecimal();
	return(c.doubleValue());
    }

    public double divide(QI t) {
	BigDecimal p=this.toBigDecimal();
	BigDecimal q=t.toBigDecimal();
	BigDecimal r=p.divide(q,BigDecimal.ROUND_UP);
	return(r.doubleValue());
    }


    /**Here are some constants*/

    public static BigDecimal PHI() {
	BigDecimal x=new BigDecimal("1.6180339887498948482045868343656381177203091798057628621354486227052604628");

	return(x);
    }

    public static double phi() {
	double p=(1+Math.sqrt(5))/2;
	return(p);
    }

    public static double innerBound() {
	double p=-Math.pow(phi(),4);
	return(p);
    }

    public static double outerBound() {
	double p=Math.pow(phi(),6);
	return(p);
    }


    public String toString() {
      String S="";
      if(a.longValue()>=0) S=S+"+";
      S=S+a.toString();
      if(b.longValue()>=0) S=S+"+";
      S=S+b.toString();
      S=S+" \u03D5";
      return(S);
    }

    public void print() {
	System.out.print(toString());
    }
    public void println() {
	print();
	System.out.println("");
    }

}



